==> gps-0.8.0.tar.gz: gps-0.8.0/src-ip.c <==
#include<libnet.h>
#include<pcap.h>
#include<stdlib.h>
#include<time.h>
#include<netinet/if_ether.h>
#include<netinet/ip.h>
#include<netinet/tcp.h>
#include "pcap-int.h"
#include "src-ip.h"

// this function TCP pings every machine on the local network
// and return the list of the host that seem down
arp_elt *get_src_ip_list(netdev *nd, u_int16_t pingport, int rseed, int *nbip, u_int32_t dstip)
{
	struct pcap *pdescr;
	struct pcap_pkthdr phdr;
	arp_elt *srcip=NULL;
	int raw_sock, host_up;
	u_int32_t ip, ip_max;
	struct timeval tv1, tv2;
	struct ip *ihdr;
	struct tcphdr *thdr;
	u_char my_packet[GPS_PACKET];
	const u_char *packet;
	char ebuf[GPS_ERRBUF_SIZE];

	*nbip=0;
	if(!(pdescr=pcap_open_live(nd->name, 65536, 1, -1, ebuf)))
		libnet_error(LIBNET_ERR_FATAL, "pcap_open_live failed (%s)\n", ebuf);
	if((raw_sock=libnet_open_raw_sock(IPPROTO_RAW))==-1)
		libnet_error(LIBNET_ERR_FATAL, "libnet_open_raw_sock: not able to open a raw socket\n");
	unblock_socket(pdescr->fd);


	for(ip=nd->ip & nd->netmask, ip_max=ip | ~nd->netmask; ntohl(ip)<ntohl(ip_max); ip+=ntohl(1))
	{
		if(!(ip & ~nd->netmask) || (ip & ~nd->netmask)==~nd->netmask || ip==nd->ip || ip==dstip) continue;
		libnet_build_tcp(libnet_get_prand(rseed)+1024, pingport, 65535*libnet_get_prand(rseed)*libnet_get_prand(rseed), 65535*libnet_get_prand(rseed)*libnet_get_prand(rseed), TH_ACK, 1024, 0, NULL, 0, my_packet+GPS_IP_H);
		libnet_build_ip(GPS_TCP_H, IPTOS_LOWDELAY, libnet_get_prand(rseed)*libnet_get_prand(rseed), 0, 255, IPPROTO_TCP, nd->ip, ip, NULL, 0, my_packet);
		if(libnet_do_checksum(my_packet, IPPROTO_TCP, GPS_TCP_H)==-1)
			{ libnet_error(LIBNET_ERR_WARNING, "libnet_do_checksum: TCP checksum failed\n"); continue; }
		if(libnet_write_ip(raw_sock, my_packet, GPS_PACKET)<GPS_PACKET)
			{ libnet_error(LIBNET_ERR_WARNING, "libnet_write_ip: not able to write to the raw socket\n"); continue; }
		gettimeofday(&tv1, NULL);
		host_up=0;
		while(!host_up) {
			if((packet=pcap_next(pdescr, &phdr))) {
				if(ntohs(((struct ether_header *)packet)->ether_type)==ETHERTYPE_IP) {
					ihdr=(struct ip *)(packet+ETHER_HDR_LEN);
					if(ihdr->ip_src.s_addr==ip && ihdr->ip_dst.s_addr==nd->ip && !(ihdr->ip_off & ~ntohs(IP_DF)) && ihdr->ip_p==IPPROTO_TCP) {
						thdr=(struct tcphdr *)((u_char *)ihdr+4*ihdr->ip_hl);
						if((thdr->th_flags & TH_RST) && ntohs(thdr->th_sport==pingport))
							host_up=1;
					}
				}
			}
			if(!host_up) {
				gettimeofday(&tv2, NULL);
				if(TIMEVAL_MSEC_DIFF(tv2, tv1)>LAN_PING_TIMEOUT) {
					/* this host didn't answer, lets assume it is down */
					if(!(srcip=(arp_elt *)realloc(srcip, (*nbip+1)*sizeof(arp_elt)))) error(NULL, -2);
					srcip[*nbip].ip=ip;
					get_ea(srcip[*nbip].ea, rseed);
					(*nbip)++;
					host_up=1;
				}
			}
		}
	}

	if(libnet_close_raw_sock(raw_sock)==-1)
		libnet_error(LIBNET_ERR_WARNING, "libnet_close_raw_sock failed\n");
	pcap_close(pdescr);
	return(srcip);
}

==> gps-0.8.1.tar.gz: gps-0.8.1/src-ip.c <==
#include<time.h>
#include<stdlib.h>
#include<libnet.h>
#include<pcap.h>
#include "pcap-int.h"
#include "src-ip.h"

// this function TCP pings every machine on the local network
// and return the list of the host that seem down
arp_elt *get_src_ip_list(netdev *nd, u_int16_t pingport, int rseed, int *nbip, u_int32_t dstip)
{
	struct pcap *pdescr;
	struct pcap_pkthdr phdr;
	arp_elt *srcip=NULL;
	int raw_sock, host_up;
	u_int32_t ip, ip_max;
	struct timeval tv1, tv2;
	struct libnet_ip_hdr *ihdr;
	struct libnet_tcp_hdr *thdr;
	u_char my_packet[GPS_PACKET];
	const u_char *packet;
	char ebuf[GPS_ERRBUF_SIZE];

	*nbip=0;
	if(!(pdescr=pcap_open_live(nd->name, 65536, 1, -1, ebuf)))
		libnet_error(LIBNET_ERR_FATAL, "pcap_open_live failed (%s)\n", ebuf);
	if((raw_sock=libnet_open_raw_sock(IPPROTO_RAW))==-1)
		libnet_error(LIBNET_ERR_FATAL, "libnet_open_raw_sock: not able to open a raw socket\n");
	unblock_socket(pdescr->fd);


	for(ip=nd->ip & nd->netmask, ip_max=ip | ~nd->netmask; ntohl(ip)<ntohl(ip_max); ip+=ntohl(1))
	{
		if(!(ip & ~nd->netmask) || (ip & ~nd->netmask)==~nd->netmask || ip==nd->ip || ip==dstip) continue;
		libnet_build_tcp(libnet_get_prand(rseed)+1024, pingport, 65535*libnet_get_prand(rseed)*libnet_get_prand(rseed), 65535*libnet_get_prand(rseed)*libnet_get_prand(rseed), TH_ACK, 1024, 0, NULL, 0, my_packet+GPS_IP_H);
		libnet_build_ip(GPS_TCP_H, IPTOS_LOWDELAY, libnet_get_prand(rseed)*libnet_get_prand(rseed), 0, 255, IPPROTO_TCP, nd->ip, ip, NULL, 0, my_packet);
		if(libnet_do_checksum(my_packet, IPPROTO_TCP, GPS_TCP_H)==-1)
			{ libnet_error(LIBNET_ERR_WARNING, "libnet_do_checksum: TCP checksum failed\n"); continue; }
		if(libnet_write_ip(raw_sock, my_packet, GPS_PACKET)<GPS_PACKET)
			{ libnet_error(LIBNET_ERR_WARNING, "libnet_write_ip: not able to write to the raw socket\n"); continue; }
		gettimeofday(&tv1, NULL);
		host_up=0;
		while(!host_up) {
			if((packet=pcap_next(pdescr, &phdr))) {
				if(ntohs(((struct libnet_ethernet_hdr *)packet)->ether_type)==ETHERTYPE_IP) {
					ihdr=(struct libnet_ip_hdr *)(packet+ETHER_HDR_LEN);
					if(ihdr->ip_src.s_addr==ip && ihdr->ip_dst.s_addr==nd->ip && !(ihdr->ip_off & ~ntohs(IP_DF)) && ihdr->ip_p==IPPROTO_TCP) {
						thdr=(struct libnet_tcp_hdr *)((u_char *)ihdr+4*ihdr->ip_hl);
						if((thdr->th_flags & TH_RST) && ntohs(thdr->th_sport==pingport))
							host_up=1;
					}
				}
			}
			if(!host_up) {
				gettimeofday(&tv2, NULL);
				if(TIMEVAL_MSEC_DIFF(tv2, tv1)>LAN_PING_TIMEOUT) {
					/* this host didn't answer, lets assume it is down */
					if(!(srcip=(arp_elt *)realloc(srcip, (*nbip+1)*sizeof(arp_elt)))) error(NULL, -2);
					srcip[*nbip].ip=ip;
					get_ea(srcip[*nbip].ea, rseed);
					(*nbip)++;
					host_up=1;
				}
			}
		}
	}

	if(libnet_close_raw_sock(raw_sock)==-1)
		libnet_error(LIBNET_ERR_WARNING, "libnet_close_raw_sock failed\n");
	pcap_close(pdescr);
	return(srcip);
}
